@@ -23,6 +23,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "main.h"
 #include <X11/Xatom.h>
 #include "gcs-i18n.h"
+#include "Util.h"
+
+using namespace Util;
 
 /**
  * Sets the background to the image in the file specified.
@@ -45,7 +48,7 @@ bool SetBG::set_bg(	Glib::ustring &disp, Glib::ustring file, SetMode mode, Gdk::
 	Glib::RefPtr<Gdk::Pixmap> pixmap;
 
 	// open display and screen
-	_display = (disp == "") ? Gdk::DisplayManager::get()->get_default_display() : Gdk::Display::open(disp);
+	_display = (disp == "") ? Gdk::Display::open(Gdk::DisplayManager::get()->get_default_display()->get_name()) : Gdk::Display::open(disp);
 	if (!_display) {
 		std::cerr << _("Could not open display") << " " << disp << "\n";
 		return false;
@@ -90,6 +93,10 @@ bool SetBG::set_bg(	Glib::ustring &disp, Glib::ustring file, SetMode mode, Gdk::
 		pixbuf->get_height(), Gdk::INTERP_NEAREST, 255, 1, bgcolor.get_pixel(),
 		bgcolor.get_pixel());
 
+    // if automatic, figure out what mode we really want
+    if (mode == SetBG::SET_AUTO)
+        mode = SetBG::get_real_mode(pixbuf, winw, winh);
+
 	switch(mode) {
 	
 		case SetBG::SET_SCALE:
@@ -104,8 +111,8 @@ bool SetBG::set_bg(	Glib::ustring &disp, Glib::ustring file, SetMode mode, Gdk::
 			outpixbuf = SetBG::make_center(pixbuf, winw, winh, bgcolor);
 			break;
 
-		case SetBG::SET_BEST:
-			outpixbuf = SetBG::make_best(pixbuf, winw, winh, bgcolor);
+		case SetBG::SET_ZOOM:
+			outpixbuf = SetBG::make_zoom(pixbuf, winw, winh, bgcolor);
 			break;
 			
 		default:
@@ -190,6 +197,8 @@ bool SetBG::set_bg_xinerama(XineramaScreenInfo* xinerama_info, gint xinerama_num
 	gint xin_screen_num; 
 	int xin_offset = -1;
 
+    program_log("set_bg_xinerama(): num screens %d, xin screen %s, file %s", xinerama_num_screens, xinerama_screen.c_str(), file.c_str());
+
 	// get specific xinerama "screen"
 	// xinerama_screen is a string that should be "xin_#"
 	// "xin_-1" refers to the whole thing
@@ -209,8 +218,8 @@ bool SetBG::set_bg_xinerama(XineramaScreenInfo* xinerama_info, gint xinerama_num
 		}
 	}
 
-	// open display and screen
-	_display = Gdk::DisplayManager::get()->get_default_display();
+	// open display and screen (make sure it is a copy of the default display)
+	_display = Gdk::Display::open(Gdk::DisplayManager::get()->get_default_display()->get_name());
 	if (!_display) {
 		std::cerr << _("Could not open display") << "\n";
 		return false;
@@ -267,13 +276,24 @@ bool SetBG::set_bg_xinerama(XineramaScreenInfo* xinerama_info, gint xinerama_num
 	// alloc our background color 
 	colormap->alloc_color(bgcolor, false, true);
 
-	if (xoldpm != NULL) {
+	if (xoldpm) {
 		// grab the old pixmap and ref it into a gdk pixmap
 		pixmap = Gdk::Pixmap::create(_display, *xoldpm);
-	
-		} else {
+		// check that this pixmap is the right size
+		int width, height;
+		pixmap->get_size(width, height);
+
+		if ((width != winw) || (height != winh) || (pixmap->get_depth() != window->get_depth()) ) {
+			XKillClient(xdisp, *((Pixmap *) data_root));
+			xoldpm = NULL;
+		}
+	}
+
+	if (!xoldpm) {
 		// we have to create it
 		pixmap = Gdk::Pixmap::create(window,winw,winh,window->get_depth());
+        int width, height;
+        pixmap->get_size(width, height);
 	}
 
 	// set the colormap 
@@ -286,14 +306,18 @@ bool SetBG::set_bg_xinerama(XineramaScreenInfo* xinerama_info, gint xinerama_num
 		std::cerr << _("ERROR: Could not load file in bg set") << ": " << e.what() << "\n";
 		return false;
 	}
-	
+
 	// apply the bg color to pixbuf here, because every make_ method would
 	// have to do it anyway.
 	pixbuf = pixbuf->composite_color_simple(pixbuf->get_width(),
 		pixbuf->get_height(), Gdk::INTERP_NEAREST, 255, 1, bgcolor.get_pixel(),
 		bgcolor.get_pixel());
 
-	switch(mode) {
+	// if automatic, figure out what mode we really want
+    if (mode == SetBG::SET_AUTO)
+        mode = SetBG::get_real_mode(pixbuf, winw, winh);
+
+    switch(mode) {
 	
 		case SetBG::SET_SCALE:
 			outpixbuf = SetBG::make_scale(pixbuf, tarw, tarh, bgcolor);
@@ -307,8 +331,8 @@ bool SetBG::set_bg_xinerama(XineramaScreenInfo* xinerama_info, gint xinerama_num
 			outpixbuf = SetBG::make_center(pixbuf, tarw, tarh, bgcolor);
 			break;
 
-		case SetBG::SET_BEST:
-			outpixbuf = SetBG::make_best(pixbuf, tarw, tarh, bgcolor);
+		case SetBG::SET_ZOOM:
+			outpixbuf = SetBG::make_zoom(pixbuf, tarw, tarh, bgcolor);
 			break;
 			
 		default:
@@ -336,6 +360,8 @@ bool SetBG::set_bg_xinerama(XineramaScreenInfo* xinerama_info, gint xinerama_num
 	window->set_back_pixmap(pixmap, false);
 	window->clear();
 
+    _display->close();
+
 	return true;
 	
 }
@@ -353,13 +379,19 @@ bool SetBG::nautilus_running(Glib::RefPtr<Gdk::Window> rootwin)
     gint format;
     gint length;
 	guchar *data;
+    gboolean retval;
 
-	return (TRUE == gdk_property_get(rootwin->gobj(), 
-								 gdk_atom_intern("NAUTILUS_DESKTOP_WINDOW_ID", FALSE),
-								 gdk_atom_intern("WINDOW", FALSE),
-								 0,
-								 4, /* wtf is a length of a window */
-								 FALSE, &type, &format, &length, &data));
+	retval = gdk_property_get(rootwin->gobj(), 
+						      gdk_atom_intern("NAUTILUS_DESKTOP_WINDOW_ID", FALSE),
+                              gdk_atom_intern("WINDOW", FALSE),
+							  0,
+							  4, /* wtf is a length of a window */
+							  FALSE, &type, &format, &length, &data);
+
+    if (retval)
+        g_free(data);
+
+    return retval;
 }
 
 /**
@@ -378,7 +410,7 @@ bool SetBG::set_bg_nautilus(Glib::RefPtr<Gdk::Screen> screen, Glib::ustring file
 		case SetBG::SET_SCALE:  strmode = "stretched";  break;
 		case SetBG::SET_TILE:   strmode = "wallpaper"; break; 
 		case SetBG::SET_CENTER: strmode = "centered"; break;
-		case SetBG::SET_BEST:   strmode = "scaled"; break;
+		case SetBG::SET_ZOOM:   strmode = "scaled"; break;
 	};
 
 	gchar** argv = (gchar**)g_malloc(sizeof(gchar*)*10);
@@ -511,14 +543,14 @@ Glib::RefPtr<Gdk::Pixbuf> SetBG::make_center(const Glib::RefPtr<Gdk::Pixbuf> ori
 }
 
 /**
- * Handles SET_BEST mode.
+ * Handles SET_ZOOM mode.
  *
  * @param	orig	The original pixbuf
  * @param	winw	Width of the window
  * @param	winh	Height of the window
  * @param	bgcolor	Background color
  */
-Glib::RefPtr<Gdk::Pixbuf> SetBG::make_best(const Glib::RefPtr<Gdk::Pixbuf> orig, const gint winw, const gint winh, Gdk::Color bgcolor) {
+Glib::RefPtr<Gdk::Pixbuf> SetBG::make_zoom(const Glib::RefPtr<Gdk::Pixbuf> orig, const gint winw, const gint winh, Gdk::Color bgcolor) {
 		
 	int x, y, resx, resy;
 	x = y = 0;
@@ -584,9 +616,12 @@ Glib::ustring SetBG::mode_to_string( const SetMode mode ) {
 		case SET_TILE:
 			ret = Glib::ustring(_("Tile"));
 			break;
-		case SET_BEST:
-			ret = Glib::ustring(_("Best"));
+		case SET_ZOOM:
+			ret = Glib::ustring(_("Zoom"));
 			break;
+        case SET_AUTO:
+            ret = Glib::ustring(_("Auto"));
+            break;
 	};
 	
 	return ret;
@@ -604,11 +639,13 @@ SetBG::SetMode SetBG::string_to_mode( const Glib::ustring str ) {
 		return SetBG::SET_CENTER;
 	else if ( str == Glib::ustring(_("Tile")) )
 		return SetBG::SET_TILE;
-	else if ( str == Glib::ustring(_("Best")) )
-		return SetBG::SET_BEST;
+	else if ( str == Glib::ustring(_("Zoom")) )
+		return SetBG::SET_ZOOM;
+    else if ( str == Glib::ustring(_("Auto")) )
+        return SetBG::SET_AUTO;
 
 	// shouldn't get here
-	return SetBG::SET_SCALE;
+	return SetBG::SET_AUTO;
 }
 
 /**
@@ -630,3 +667,31 @@ guint32 SetBG::GdkColorToUint32(const Gdk::Color col)
 	return ret;
 }
 
+/**
+ * Determines the best set mode for the pixbuf based on its size relative to
+ * the window size.
+ *
+ * @param   pixbuf  The loaded pixbuf from the file, before it has been sized.
+ * @param   width   The width of the root window.
+ * @param   height  The height of the root window.
+ */
+SetBG::SetMode SetBG::get_real_mode(const Glib::RefPtr<Gdk::Pixbuf> pixbuf, const gint width, const gint height)
+{
+    SetBG::SetMode mode = SetBG::SET_ZOOM;
+    float ratio = ((float)pixbuf->get_width()) / ((float)pixbuf->get_height());
+
+    float f2t = 1.333f;
+    float f2f = 1.25f;
+
+    if (fabsf(ratio - f2t) < 0.001)
+        mode = SetBG::SET_SCALE;
+    else if (fabsf(ratio - f2f) < 0.001)
+        mode = SetBG::SET_SCALE;
+    else if (ratio == 1.0 && pixbuf->get_width() <= 640)
+        mode = SetBG::SET_TILE;
+    else if (pixbuf->get_width() <= width && pixbuf->get_height() <= height)
+        mode = SetBG::SET_CENTER;
+
+    return mode;
+}
+
